;; Define breeds
breed [robots robot]

robots-own [

  ;; ...
]

patches-own [
  dirt-level                     ;; All the patches will get a certain level of dirt between 0 and 100
  previous_dirt-level            ;; Dirt-level of the previous time step
  hard_to_clean                  ;; Denotes how hard it is to clean a patch (1 means that the cleaning hardness of the patch is normal, a number lower than 1 means that the patch is relatively hard to clean)
  mindirt                        ;; Minimal dirtiness value for the overall dirt
  maxdirt                        ;; Maximal dirtiness value for the overall dirt
  mindirt-cluster                ;; Minimal dirtiness value for the dirt clusters
  maxdirt-cluster                ;; Maximal dirtiness value for the dirt clusters
  timescleaned                   ;; Denotes how often a patch is cleaned by the robot
]

globals [
  t_turn_top                     ;; Time necessary for the first turn at the top of the window [s]
  t_turn_corner                  ;; Time necessary for making a 180 degree turn from a corner position [s]
  t_turn                         ;; Time necessary for making a 180 degree turn from a position at the window edge [s]
  t_changerow                    ;; Time necessary to change row from a corner position [s]
  t_changerow_corner             ;; Time necessary to change row frow a position at the window edge [s]
  t_pointturn                    ;; Time necessary to make a 180-degree turn when there are no obstructions present (far enough from the window edges) [s]
  t_factor                       ;; Factor for scaling the waiting time, so that the simulation is faster than the real life situation
  endfirstpartmotion             ;; Variable that is used to check if the first part of the motion, namely moving to the upper left corner of the window, is over
  mindirtred                     ;; Minimal dirt reduction
  maxdirtred                     ;; Maximal dirt reduction
  x_start                        ;; Start x coordinate
  rowclean                       ;; variable denoting if a row is clean [0 = dirty, 1 = clean]
  patches-to-check               ;; Set of patches that need to be checked for cleanliness
  considered-dirty-patches       ;; The set of patches with a dirt-level greater than 0 that is included in the set patches-to-check
  n                              ;; Rownumber
  n_max                          ;; Maximum row number
  n_min                          ;; Minimum row number
  v                              ;; Speed of the robot [m/s]
  s_turn_corner                  ;; Distance that needs to be traveled to be able to turn when starting from a corner position [m]
  s_turn                         ;; Distance that needs to be traveled to be able to turn when starting from a window edge position [m]
  s_changerow                    ;; Distance that needs to be traveled to change row from a postion that is not in the top or bottom row of the window [m]
  s_changerow_corner             ;; Distance that needs to be traveled to change row when the change row motion starts from a corner postion [m]
  omega                          ;; Rotation speed [deg/s]
  t                              ;; Cleaning time (the time starts at the instant that the robot starts moving from the position in the middle of the window) [s]
  powerconsumption               ;; Average powerconsumption when the robot is not cleaning [W]
  powerconsumption_cleaning      ;; Average powerconsumption when the robot is cleaning [W]
  waterconsumption               ;; Average waterconsumption while the robot is cleaning [L/s]
  energy                         ;; Variable denoting the amount of energy used by the robot since the beginning of the cleaning task [J]
  water                          ;; Variable denoting the amount of water used by the robot since the beginning of the cleaning task [L]
  stop_robot                     ;; Variable that denotes if the robot has finished the cleaning task
  justswitchedrow                ;; Checks if the robot has just switched rows and has not yet reached one of the ends of the new row
  timesrowcleaned                ;; Denotes how much times the robot has moved from the one end of the current row to the other end of the current row
  checkzigzag                    ;; Checks if the zigzag movement has been used by the robot while going from one end of the current row to the other end of the currrent row
  checkturndirt                  ;; Checks if the turndirt movement has been used by the robot while going from one end of the current row to the other end of the currrent row
  xcor_min                       ;; The smallest x-coordinate (allong the width of the window) from the set considered-dirty-patches
  xcor_max                       ;; The largest x-coordinate (allong the width of the window) from the set considered-dirty-patches
]

to setup-window
  clear-all                                                                        ;; Resets the world to an initial, empty state
  resize-world 0 (((width-window * 100) / 5) - 1) 0 (((height-window * 100) / 5) - 1)          ;; Changes the number of patches based on the set window dimensions, each patch has dimensions 5 x 5 cm. Note that the origin is placed at bottom left patch of the window.
  reset-ticks                                                                      ;; Resets the tick counter.
  adjust-patch-size                                                                ;; Calls procedure 'adjust-patch-size', needed to show the entire window on your screen
  ask patches [
    set pcolor 86                                                                  ;; Makes all the patches blue
  ]
end

to adjust-patch-size
  ;; This code is only used to show the entire window nicely on your screen
  if height-window > 2.5 [                                   ;; When the height of the window is bigger than 2.5 m, the patch size is changed to make the window fit nicely on the screen
    set-patch-size 10
  ]
  if height-window > 3.5 [                                   ;; When the height of the window is bigger than 3.5 m, the patch size is changed to make the window fit nicely on the screen
    set-patch-size 8
  ]
  if (height-window < 2.5) and (width-window < 2.5) [        ;; When both the width and height are smaller than 2.5 m, the patch size is changed to make the window fit nicely on the screen
    set-patch-size 14
  ]
end

to setup-dirt  ;; Sets up the dirt on the window
  ask patches [
    set mindirt 10  ;; Set mindirt to 10
    set maxdirt 30  ;; Set maxdirt to 30
    set mindirt-cluster 40  ;; Set mindirt-cluster to 40
    set maxdirt-cluster 100  ;; Set maxdirt-cluster to 100
    set dirt-level (dirt-intensity / 100) * (mindirt + random (maxdirt - mindirt))  ;; Every patch gets a random dirtiness value between mindirt and maxdirt which is multiplied by the intensity of the dirt
    ifelse hard_to_clean_active [  ;; When the hard_to_clean_active switch is switched on:
      set hard_to_clean 100 * (random-exponential 1)  ;; Set the hard_to_clean value of each patch to a value from an exponential distribution with mean 1 and multiply it with 100
      if hard_to_clean > 1 [  ;; If a patch has an hard_to_clean value that is higher than 1, it is set to 1
        set hard_to_clean 1
      ]
    ][
      set hard_to_clean 1  ;; If the hard_to_clean_active switch is switched off, all patches get a hard_to_clean value of 1
    ]
  ]
  repeat nr-of-dirt-clusters [make-cluster]  ;; Make dirt clusters with higher dirt-levels, the amount of dirt clusters made is the amount that is specified by the slider nr_of_dirt_clusters
  ask patches [
    set previous_dirt-level dirt-level  ;; Set the value of previous_dirt-level of all to patches to the current dirt-level of the patch
    ;; Colors assigned to the dirty patches that correspond with their dirt-level. Color gets darker when the dirt-level increases
    if (dirt-level > 80) and (dirt-level <= 100) [
      set pcolor 31
    ]
    if dirt-level > 60 and dirt-level <= 80 [
      set pcolor 33
    ]
    if dirt-level > 40 and dirt-level <= 60 [
      set pcolor 35
    ]
    if dirt-level > 20 and dirt-level <= 40 [
      set pcolor 37
    ]
    if dirt-level > 0 and dirt-level <= 20 [
      set pcolor 39
    ]
  ]
  export-world "window.csv"  ;; Save the just generated dirt-configuration together with other settings in the file window.csv
end

to make-cluster  ;; Make a dirt cluster with higher dirtiness levels
    loop [
      let cluster [patches in-radius (random-float 1.5)] of one-of patches  ;; Select a patch and the patches in a random radius between 0 and 1.5 of that patch and assign this patch set to cluster
      if all? (patch-set [neighbors] of cluster) [dirt-level <= mindirt-cluster] [  ;; Check if the neighbours of the patch set cluster do not have higher dirt-levels (This prevents clusters from spawning on top of eachother)
        ask cluster [set dirt-level (dirt-intensity / 100) * (mindirt-cluster + random (maxdirt-cluster - mindirt-cluster)) ]  ;; Reset the dirt-levels of the patches in cluster with a random value between mindirt-cluster and maxdirt-cluster multiplied by the intensity of the dirt
        stop
      ]
    ]
end

to updatedirt    ;; At most time steps, the dirt-level of some patches changes. This method is used to assign colors to all patches based on the new dirt-levels and reset the dirt-level back to zero if it is below zero
  ask patches[
    if dirt-level < 0 [
      set dirt-level 0
    ]
    if (dirt-level > 80) and (dirt-level <= 100) [
      set pcolor 31
    ]
    if dirt-level > 60 and dirt-level <= 80 [
      set pcolor 33
    ]
    if dirt-level > 40 and dirt-level <= 60 [
      set pcolor 35
    ]
    if dirt-level > 20 and dirt-level <= 40 [
      set pcolor 37
    ]
    if dirt-level > 0 and dirt-level <= 20 [
      set pcolor 39
    ]
    if dirt-level = 0 [
      set pcolor 86
    ]
  ]
end

to setup-robot  ;; Sets up the robot and most of the used variables
  clear-turtles                                 ;; If there is already a robot, it is removed
  create-robots 1 [                     ;; Creates a (new) robot
    set size 5                          ;; Robot gets a size of 5 x 5 patches which equals 25 x 25 cm
    ifelse (height-window / 0.05) mod 2 > 0 [  ;; This piece of code ensures that the robot is placed in the middle of a patch in the middle of the window
      set ycor (height-window / 0.05 / 2) - 0.5 ][
      set ycor (height-window / 0.05 / 2)
    ]
    ifelse (width-window / 0.05) mod 2 > 0 [
      set xcor (width-window / 0.05 / 2) - 0.5 set x_start xcor][
      set xcor (width-window / 0.05 / 2) set x_start xcor
    ]
  ]
  set-default-shape robots "windowcleaner"  ;; Gives the robot a customized square shape
  set mindirtred 30  ;; Minimum dirt reduction is set to 30
  set maxdirtred 60  ;; Maximal dirt reduction is set to 60
  set n (height-window / 0.05) / 5  ;; Calculate number of rows and assign this value to n
  set s_turn_corner 0.3303  ;; Set s_turn_corner to 0.3303
  set s_turn 0.0518  ;; Set s_turn to 0.0518
  set s_changerow 0.25  ;; Set s_changerow to 0.25
  set s_changerow_corner 0.2  ;; Set s_changerow_corner to 0.2
  set n_max (height-window / 0.05) / 5 ;; Calculate the maximum row number and assign this value to n_max
  set n_min 1  ;; The minimum row number is 1
  set v 0.1  ;; Set v to 0.1
  set omega 63.6  ;; Set omega to 63.6
  set powerconsumption 50  ;; Set powercosumption to 50
  set powerconsumption_cleaning 73.5  ;; Set powerconsumption_cleaning 73.5
  set waterconsumption 1 / 8640  ;; Set watercosumption to 0.1
  set t_factor 100  ;; set t_factor to 100
  set t_turn_top ((s_turn / v) + (45 / omega) + (s_turn_corner / v)) / t_factor  ;; Calculate the time necessary for the first turn at the top of the window, devide it by t_factor and assign it to t_turn_top
  set t_turn_corner (((2 * s_turn_corner) / v) + (180 / omega)) / t_factor  ;; Calculate the time necessary for making a 180 degree turn from a corner position, devide it by t_factor and assign it to t_turn_corner
  set t_turn (((2 * s_turn) / v) + (180 / omega)) / t_factor  ;; Calculate the time necessary for making a 180 degree turn from a position at the window edge, devide it by t_factor and assign it to t_turn
  set t_changerow ((s_turn / v) + (90 / omega) + (s_changerow / v) + (90 / omega)) / t_factor  ;; Calculate the time necessary to change row frow a position at the window edge, devide it by t_factor and assign it to t_changerow
  set t_changerow_corner ((s_turn_corner / v) + (45 / omega) + (s_changerow_corner / v) + (90 / omega)) / t_factor  ;; Calculate the time necessary to change row from a corner position, devide it by t_factor and assign it to t_changerow_corner
  set t_pointturn (180 / omega) / t_factor  ;; Calculate the time necessary to make a 180-degree turn when no obstructions are in the way (far enough away fromt the window edge), devide it by t_factor and assign it to t_pointturn
  set xcor_min 2  ;; Set xcor_min to 2 to ensure that the robot first cleans the entire row when the turndirt motion planning algorithm is used
  set xcor_max ((width-window / 0.05) - 3) ;; Set xcor_max to ((width-window / 0.05) - 3) to ensure that the robot first cleans the entire row when the turndirt motion planning algorithm is used
end

to clean-window  ;; Clean the window
  if stop_robot = 0 [  ;; If the robot has not yet cleaned the entire window it has to continue
    ifelse endfirstpartmotion != 0  ;; Check if the robot has moved to the top left corner for the first time
    [motion-planning]  ;; If this is the case, start with the selected motion planning algorithm
    [move-startposition]  ;; If this is not the case, the robot has to continue with moving to the top left corner
  wait (0.05 / v) / t_factor  ;; Wait the time that it takes the robot to move one patch devided by t_factor
  set t (t + (0.05 / v))  ;; Add the time that the robot takes to move one patch to the cleaning time
  if select_motion_planning = "zigzag" or select_motion_planning = "turndirt" [  ;; When the selected motion planning algorithm is zigzag or turndirt,
    set energy (energy + powerconsumption * (0.05 / v))  ;; Add the energy necessary to move one patch while cleaning is not active to the total consumed energy
  ]
  if select_motion_planning = "standard" [  ;; When the selected motion planning algorithm is standard,
    set energy (energy + powerconsumption_cleaning * (0.05 / v))  ;; Add the energy necessary to move one patch while cleaning to the total consumed energy
    set water (water + waterconsumption * (0.05 / v))  ;; Add the water used to clean one patch once to the total water consumption
  ]
  tick  ;; Advances the tick counter with one tick
  ]
end

to move-startposition  ;; This method ensures that the robot first moves to the top left corner of the window (the start position of the motion planning algorithm)
  ask robots[
     if ycor < ((height-window / 0.05) - 3)[     ;; If the robot has not reached the top of the window, it has to move to the top
       set heading 0  ;; Set the heading direction to the top of the window
       forward 1  ;; Move forward one patch
       set energy (energy + powerconsumption * (0.05 / v)) ;; Add the energy necessary to move one patch while cleaning is not active to the total consumed energy
    ]
     if ycor = ((height-window / 0.05) - 3) and xcor = x_start[     ;; If the robot has reached the top of the window, it has to wait t_turntop to account for the turning time and head to the left side of the window
       wait t_turn_top
       set t (t + t_factor * t_turn_top)  ;; Add the time necessary to turn at the top of the window to the cleaning time
       set energy (energy + powerconsumption * t_factor * t_turn_top)  ;; Add the consumed energy from the turn at the top of the window to the total consumed energy
       set xcor xcor - 3  ;; The robot is placed three patches to the right after turning to account for the fact that the robot does not end up at the starting spot of the turn after turning
       set heading 270  ;; Set the heading direction to the left
       forward 1  ;; Move forward one patch
    ]
    if ycor = ((height-window / 0.05) - 3) and xcor != x_start and xcor > 2[     ;; If the robot is moving to the left and has not reached the corner, it has to continue moving to the left
      forward 1  ;; Move forward one patch
      clean  ;; The robot already cleans patches, since it is already in the top row
      set energy (energy + powerconsumption_cleaning * (0.05 / v))  ;; Add the energy necessary to move one patch while cleaning to the total consumed energy
    ]
    if ycor = ((height-window / 0.05) - 3) and xcor = 2[    ;; If the robot has reached the top left corner, it has to wait t_turn_corner to account for the turning time and set its heading direction to the right side of the window
      wait t_turn_corner
      set t (t + t_factor * t_turn_corner)  ;; Add the time necessary to turn at the top left corner of the window to the cleaning time
      set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the top left corner of the window to the total consumed energy
      set heading 90  ;; Set the heading direction to the right
      set endfirstpartmotion 1  ;; The robot has reached the top left corner, it thus has to continue with the selected motion planning algorithm
    ]
  ]
end

to motion-planning  ;; The motion planning algorithm (selected from the three possiblities)
  if select_motion_planning = "zigzag" [  ;; Check if the zigzag motion planning algorithm is selected
    ask robots[
      forward 1  ;; During the motion, the robot has to conitnuously move forward and clean patches
      clean
      check  ;; Furthermore, it has to continuously check if a dirty patch passes the dirt detect sensors. If this is the case, it has to initiate the reciprocating motion
    ]
    updatedirt  ;; Update the colors of the patches based on the new dirt-levels and set negative dirt-levels back to zero
    ask robots [
      if xcor = ((width-window / 0.05) - 3) and heading = 90 [   ;; Check if the robot has reached the right end of the window and is heading to the right
        if justswitchedrow = 0 [  ;; If the robot has not just switched rows, increment the variable timesrowcleaned with 1
          set timesrowcleaned timesrowcleaned + 1
        ]
        ifelse n = n_max or n = n_min [  ;; If the robot is located at the top or bottom row, it takes longer to turn (When changing rows, the robot still has to turn first to check if the last bit of the row has been cleaned properly)
          wait t_turn_corner
          set t (t + t_factor * t_turn_corner)  ;; Add the corner turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the corner of the window to the total consumed energy
        ][
          wait t_turn
          set t (t + t_factor * t_turn)  ;; Add the turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn)  ;; Add the consumed energy from the turn to the total consumed energy
        ]
        checkcleanliness  ;; The cleanliness of all patches in the current row is checked
        if ycor = 2 and rowclean = 1 [  ;; If the robot is located in the bottom row and this row is considered clean, the robot has finished the cleaning task
          set stop_robot 1
          set heading 270  ;; Set the heading direction to the heading direction after turning
        ]
        if stop_robot = 0 [  ;; Check if the robot has not yet finsihed the cleaning task
          ifelse rowclean = 1 and (checkzigzag = 0)[  ;; If the current row is clean and no zigzag movement has been initiated during the last time that the row was cleaned, the robot has to wait to account for the time to change row and move to the next row (and update cleaning time and consumed energy)
            ifelse n = n_max [  ;; If the robot has to change row and is located at one of the corners in the top row, it takes longer to change row (the bottom row is not taken into account here since the robot does not have to change row anymore if it is located in the bottom row)
              wait  (t_changerow_corner)
              set t (t + t_factor * (t_changerow_corner))  ;; Add the corner change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow_corner)  ;; Add the consumed energy from the changing of rows from a corner position to the total consumed energy
            ][
              wait (t_changerow)
              set t (t + t_factor * (t_changerow))  ;; Add the change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow)  ;; Add the consumed energy from the changing of rows to the total consumed energy
            ]
            set ycor ycor - 5  ;; Move the robot to the next row
            set xcor xcor - 5
            set n n - 1   ;; The patches of the next row need to be considered by the checkcleanliness method. Therfore, the row number is reduced by 1
            set rowclean 0  ;; Reset the variable that denotes the cleanliness of the current row
            set timesrowcleaned 0  ;; Set the times that the row has been cleaned back to 0
            set justswitchedrow 1  ;; Denotes that the robot has just switched rows
            set checkzigzag 0  ;; Set the variable that checks if a zigzag motion has been initiated during the last time that the row was cleaned back to 0
          ][  ;; If the current row is not yet clean, the robot has to turn around
            set heading 270
            set justswitchedrow 0  ;; Denotes that the robot has reached one end of the next row of the window after changing rows (This is in fact set to 0 every time that the robot reaches an end of the row, but of course it has only effect when the previous value was 1)
            set checkzigzag 0  ;; Set the variable that checks if a zigzag motion has been initiated during the last time that the row was cleaned back to 0
          ]
        ]
      ]
      if xcor = 2 and heading = 270 [  ;; Check if the robot has reached the left end of the window and is heading to the left
        if justswitchedrow = 0 [  ;; If the robot has not just switched rows, increment the variable timesrowcleaned with 1
          set timesrowcleaned timesrowcleaned + 1
        ]
        ifelse n = n_max or n = n_min [  ;; If the robot is located at the top or bottom row, it takes longer to turn (When changing rows, the robot still has to turn first to check if the last bit of the row has been cleaned properly)
          wait t_turn_corner
          set t (t + t_factor * t_turn_corner)  ;; Add the corner turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the corner of the window to the total consumed energy
        ][
          wait t_turn
          set t (t + t_factor * t_turn)  ;; Add the turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn)  ;; Add the consumed energy from the turn to the total consumed energy
        ]
        checkcleanliness  ;; The cleanliness of all patches in the current row is checked
        if ycor = 2 and rowclean = 1 [  ;; If the robot is located in the bottom row and this row is clean, the robot has finished the cleaning task
          set stop_robot 1
          set heading 90  ;; Set the heading direction to the heading direction after turning
        ]
        if stop_robot = 0 [  ;; Check if the robot has not yet finsihed the cleaning task
          ifelse rowclean = 1 and (checkzigzag = 0)[  ;; If the current row is clean and no zigzag movement has been initiated during the last time that the row was cleaned, the robot has to wait to account for the time to change row and move to the next row (and update cleaning time and consumed energy)
            ifelse n = n_max [  ;; If the robot has to change row and is located at one of the corners in the top row, it takes longer to change row (the bottom row is not taken into account here since the robot does not have to change row anymore if it is located in the bottom row)
              wait (t_changerow_corner)
              set t (t + t_factor * (t_changerow_corner))  ;; Add the corner change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow_corner)  ;; Add the consumed energy from the changing of rows from a corner position to the total consumed energy
            ][
              wait (t_changerow)
              set t (t + t_factor * (t_changerow))  ;; Add the change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow)  ;; Add the consumed energy from the changing of rows to the total consumed energy
            ]
            set ycor ycor - 5  ;; Move the robot to the next row
            set xcor xcor + 5
            set n n - 1   ;; The patches of the next row need to be considered by the checkcleanliness method. Therfore, the row number is reduced by 1
            set rowclean 0  ;; Reset the variable that denotes the cleanliness of the current row
            set timesrowcleaned 0  ;; Set the times that the row has been cleaned back to 0
            set justswitchedrow 1  ;; Denotes that the robot has just switched rows
            set checkzigzag 0  ;; Set the variable that checks if a zigzag motion has been initiated during the last time that the row was cleaned back to 0
          ][  ;; If the current row is not yet clean, the robot has to turn around
            set heading 90
            set justswitchedrow 0  ;; Denotes that the robot has reached one end of the next row of the window after changing rows (This is in fact set to 0 every time that the robot reaches an end of the row, but of course it has only effect when the previous value was 1)
            set checkzigzag 0  ;; Set the variable that checks if a zigzag motion has been initiated during the last time that the row was cleaned back to 0
          ]
        ]
      ]
    ]
  ]
  if select_motion_planning = "standard" [
    ask robots[
      forward 1  ;; During the motion, the robot has to conitnuously move forward and clean patches
      clean
    ]
    updatedirt  ;; Update the colors of the patches based on the new dirt-levels and set negative dirt-levels back to zero
    ask robots [
      if xcor = ((width-window / 0.05) - 3) and heading = 90 [   ;; Check if the robot has reached the right end of the window and is heading to the right
        ifelse n = n_max or n = n_min [  ;; If the robot is located at the top or bottom row, it takes longer to turn (When changing rows, the robot still has to turn first to check if the last bit of the row has been cleaned properly)
          wait t_turn_corner
          set t (t + t_factor * t_turn_corner)  ;; Add the corner turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the corner of the window to the total consumed energy
        ][
          wait t_turn
          set t (t + t_factor * t_turn)  ;; Add the turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn)  ;; Add the consumed energy from the turn to the total consumed energy
        ]
        checkcleanliness  ;; The cleanliness of all patches in the current row is checked
        if ycor = 2 and rowclean = 1 [  ;; If the robot is located in the bottom row and this row is clean, the robot has finished the cleaning task
          set stop_robot 1
          set heading 270  ;; Set the heading direction to the heading direction after turning
        ]
        if stop_robot = 0 [  ;; Check if the robot has not yet finsihed the cleaning task
          ifelse rowclean = 1 [  ;; If the current row is clean, the robot has to wait to account for the time to change row and move to the next row (and update cleaning time and consumed energy)
            ifelse n = n_max [  ;; If the robot has to change row and is located at one of the corners in the top row, it takes longer to change row (the bottom row is not taken into account here since the robot does not have to change row anymore if it is located in the bottom row)
              wait  (t_changerow_corner)
              set t (t + t_factor * (t_changerow_corner))  ;; Add the corner change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow_corner)  ;; Add the consumed energy from the changing of rows from a corner position to the total consumed energy
            ][
              wait (t_changerow)
              set t (t + t_factor * (t_changerow))  ;; Add the change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow)  ;; Add the consumed energy from the changing of rows to the total consumed energy
            ]
            set ycor ycor - 5  ;; Move the robot to the next row
            set xcor xcor - 5
            set n n - 1   ;; The patches of the next row need to be considered by the checkcleanliness method. Therfore, the row number is reduced by 1
            set rowclean 0  ;; Reset the variable that denotes the cleanliness of the current row
          ][  ;; If the current row is not yet clean, the robot has to turn around
            set heading 270
          ]
        ]
      ]
      if xcor = 2 and heading = 270 [  ;; Check if the robot has reached the left end of the window and is heading to the left
        ifelse n = n_max or n = n_min [  ;; If the robot is located at the top or bottom row, it takes longer to turn (When changing rows, the robot still has to turn first to check if the last bit of the row has been cleaned properly)
          wait t_turn_corner
          set t (t + t_factor * t_turn_corner)  ;; Add the corner turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the corner of the window to the total consumed energy
        ][
          wait t_turn
          set t (t + t_factor * t_turn)  ;; Add the turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn)  ;; Add the consumed energy from the turn to the total consumed energy
        ]
        checkcleanliness  ;; The cleanliness of all patches in the current row is checked
        if ycor = 2 and rowclean = 1 [  ;; If the robot is located in the bottom row and this row is clean, the robot has finished the cleaning task
          set stop_robot 1
          set heading 90  ;; Set the heading direction to the heading direction after turning
        ]
        if stop_robot = 0 [  ;; Check if the robot has not yet finsihed the cleaning task
          ifelse rowclean = 1 [  ;; If the current row is clean, the robot has to wait to account for the time to change row and move to the next row (and update cleaning time and consumed energy)
            ifelse n = n_max [  ;; If the robot has to change row and is located at one of the corners in the top row, it takes longer to change row (the bottom row is not taken into account here since the robot does not have to change row anymore if it is located in the bottom row)
              wait (t_changerow_corner)
              set t (t + t_factor * (t_changerow_corner))  ;; Add the corner change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow_corner)  ;; Add the consumed energy from the changing of rows from a corner position to the total consumed energy
            ][
              wait (t_changerow)
              set t (t + t_factor * (t_changerow))  ;; Add the change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow)  ;; Add the consumed energy from the changing of rows to the total consumed energy
            ]
            set ycor ycor - 5  ;; Move the robot to the next row
            set xcor xcor + 5
            set n n - 1   ;; The patches of the next row need to be considered by the checkcleanliness method. Therfore, the row number is reduced by 1
            set rowclean 0  ;; Reset the variable that denotes the cleanliness of the current row
          ][  ;; If the current row is not yet clean, the robot has to turn around
            set heading 90
          ]
        ]
      ]
    ]
  ]
  if select_motion_planning = "turndirt" [
    ask robots[
      forward 1  ;; During the motion, the robot has to conitnuously move forward and clean patches
      clean
    ]
    updatedirt  ;; Update the colors of the patches based on the new dirt-levels and set negative dirt-levels back to zero
    ask robots [
      if xcor = ((width-window / 0.05) - 3) and heading = 90 [   ;; Check if the robot has reached the right end of the window and is heading to the right
        ifelse n = n_max or n = n_min [  ;; If the robot is located at the top or bottom row, it takes longer to turn (When changing rows, the robot still has to turn first to check if the last bit of the row has been cleaned properly)
          wait t_turn_corner
          set t (t + t_factor * t_turn_corner)  ;; Add the corner turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the corner of the window to the total consumed energy
        ][
          wait t_turn
          set t (t + t_factor * t_turn)  ;; Add the turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn)  ;; Add the consumed energy from the turn to the total consumed energy
        ]
        checkcleanliness  ;; The cleanliness of all patches in the current row is checked
        if ycor = 2 and rowclean = 1 [  ;; If the robot is located in the bottom row and this row is clean, the robot has finished the cleaning task
          set stop_robot 1
          set heading 270  ;; Set the heading direction to the heading direction after turning
        ]
        if stop_robot = 0 [  ;; Check if the robot has not yet finsihed the cleaning task
          ifelse rowclean = 1 and checkturndirt = 0 [  ;; If the current row is clean and no turndirt movement has been initiated during the last time that the row was cleaned, the robot has to wait to account for the time to change row and move to the next row (and update cleaning time and consumed energy)
            ifelse n = n_max [  ;; If the robot has to change row and is located at one of the corners in the top row, it takes longer to change row (the bottom row is not taken into account here since the robot does not have to change row anymore if it is located in the bottom row)
              wait  (t_changerow_corner)
              set t (t + t_factor * (t_changerow_corner))  ;; Add the corner change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow_corner)  ;; Add the consumed energy from the changing of rows from a corner position to the total consumed energy
            ][
              wait (t_changerow)
              set t (t + t_factor * (t_changerow))  ;; Add the change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow)  ;; Add the consumed energy from the changing of rows to the total consumed energy
            ]
            set ycor ycor - 5  ;; Move the robot to the next row
            set xcor xcor - 5
            set n n - 1   ;; The patches of the next row need to be considered by the checkcleanliness method. Therfore, the row number is reduced by 1
            set rowclean 0  ;; Reset the variable that denotes the cleanliness of the current row
            set xcor_min 2  ;; Reset the values for xcor_min and xcor_max to 2 respectively ((width-window / 0.05) - 3) to ensure that the robot first cleans the entire new row when the turndirt motion planning algorithm is used
            set xcor_max ((width-window / 0.05) - 3)
            set checkturndirt 0  ;; Set the variable that checks if a turndirt motion has been initiated during the last time that the row was cleaned back to 0
          ][  ;; If the current row is not yet clean, the robot has to turn around
            set heading 270
            set checkturndirt 0  ;; Set the variable that checks if a turndirt motion has been initiated during the last time that the row was cleaned back to 0
          ]
        ]
      ]
      if xcor = 2 and heading = 270 [  ;; Check if the robot has reached the left end of the window and is heading to the left
        ifelse n = n_max or n = n_min [  ;; If the robot is located at the top or bottom row, it takes longer to turn (When changing rows, the robot still has to turn first to check if the last bit of the row has been cleaned properly)
          wait t_turn_corner
          set t (t + t_factor * t_turn_corner)  ;; Add the corner turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn_corner)  ;; Add the consumed energy from the turn at the corner of the window to the total consumed energy
        ][
          wait t_turn
          set t (t + t_factor * t_turn)  ;; Add the turning time to the cleaning time
          set energy (energy + powerconsumption * t_factor * t_turn)  ;; Add the consumed energy from the turn to the total consumed energy
        ]
        checkcleanliness  ;; The cleanliness of all patches in the current row is checked
        if ycor = 2 and rowclean = 1 [  ;; If the robot is located in the bottom row and this row is clean, the robot has finished the cleaning task
          set stop_robot 1
          set heading 90  ;; Set the heading direction to the heading direction after turning
        ]
        if stop_robot = 0 [  ;; Check if the robot has not yet finsihed the cleaning task
          ifelse rowclean = 1 and checkturndirt = 0 [  ;; If the current row is clean and no turndirt movement has been initiated during the last time that the row was cleaned, the robot has to wait to account for the time to change row and move to the next row (and update cleaning time and consumed energy)
            ifelse n = n_max [  ;; If the robot has to change row and is located at one of the corners in the top row, it takes longer to change row (the bottom row is not taken into account here since the robot does not have to change row anymore if it is located in the bottom row)
              wait (t_changerow_corner)
              set t (t + t_factor * (t_changerow_corner))  ;; Add the corner change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow_corner)  ;; Add the consumed energy from the changing of rows from a corner position to the total consumed energy
            ][
              wait (t_changerow)
              set t (t + t_factor * (t_changerow))  ;; Add the change row time to the cleaning time
              set energy (energy + powerconsumption * t_factor * t_changerow)  ;; Add the consumed energy from the changing of rows to the total consumed energy
            ]
            set ycor ycor - 5  ;; Move the robot to the next row
            set xcor xcor + 5
            set n n - 1   ;; The patches of the next row need to be considered by the checkcleanliness method. Therfore, the row number is reduced by 1
            set rowclean 0  ;; Reset the variable that denotes the cleanliness of the current row
            set xcor_min 2  ;; Reset the values for xcor_min and xcor_max to 2 respectively ((width-window / 0.05) - 3) to ensure that the robot first cleans the entire new row when the turndirt motion planning algorithm is used
            set xcor_max ((width-window / 0.05) - 3)
            set checkturndirt 0  ;; Set the variable that checks if a turndirt motion has been initiated during the last time that the row was cleaned back to 0
          ][  ;; If the current row is not yet clean, the robot has to turn around
            set heading 90
            set checkturndirt 0  ;; Set the variable that checks if a turndirt motion has been initiated during the last time that the row was cleaned back to 0
          ]
        ]
      ]
      if ((xcor_max - xcor_min) <= 5) and (xcor_max > 2) and (xcor_min < ((width-window / 0.05) - 3))[  ;; If the difference between xcor_min and xcor_max is smaller than or equal to the size of the robot and xcor_max is bigger than 2 and xcor_min is smaller than ((width-window / 0.05) - 3)
        set xcor_min (xcor_min - 3)  ;; Set xcor_min to xcor_min - 3 (This ensures that the robot does not get stuck, otherwise the cleaning section would be unable to reach the dirty patches)
        set xcor_max (xcor_max + 3)  ;; Set xcor_max to xcor_max + 3
      ]
      if (xcor = xcor_max) and (xcor_max > 2) and (xcor_max <= ((width-window / 0.05) - 5)) and (n != n_max) and (n != n_min) and (heading = 90) [  ;; If the robot reaches xcor_max and xcor_max is greater than 2 and smaller than or equal to ((width-window / 0.05) - 5)) (If this is not the case, then it would be harder to turn for the robot and thus the turning time would be different) and the robot is not located in the top or bottom row and is heading to the right
        wait t_pointturn  ;; Wait to account for the turning
        set t (t + t_factor * t_pointturn)  ;; Add the turning time to the cleaning time
        set energy (energy + powerconsumption * t_factor * t_pointturn)  ;; Add the consumed energy from the turn to the total consumed energy
        set heading 270  ;; Set the heading direction to the left
        checkcleanliness  ;; Check the cleanliness of the current row (Although the robot hasn't covered the entire row in the last motion, it is still permissible to use this method here since the other part of the row, that is not covered, is already known to be clean)
        set checkturndirt 1  ;; Set the variable that checks if a turndirt motion has been initiated during the last time that the row was cleaned to 1
      ]
      if (xcor = xcor_min) and (xcor_min >= 4) and (xcor_min < ((width-window / 0.05) - 3)) and (n != n_max) and (n != n_min) and (heading = 270) [  ;; If the robot reaches xcor_min and xcor_min is greater than or equal to 4 and smaller than ((width-window / 0.05) - 3)) (If this is not the case, then it would be harder to turn for the robot and thus the turning time would be different) and the robot is not located in the top or bottom row and is heading to the left
        wait t_pointturn  ;; Wait to account for the turning
        set t (t + t_factor * t_pointturn)  ;; Add the turning time to the cleaning time
        set energy (energy + powerconsumption * t_factor * t_pointturn)  ;; Add the consumed energy from the turn to the total consumed energy
        set heading 90  ;; Set the heading direction to the right
        checkcleanliness  ;; Check the cleanliness of the current row (Although the robot hasn't covered the entire row in the last motion, it is still permissible to use this method here since the other part of the row, that is not covered, is already known to be clean)
        set checkturndirt 1  ;; Set the variable that checks if a turndirt motion has been initiated during the last time that the row was cleaned to 1
      ]
    ]
  ]
end

to checkcleanliness  ;; Check the cleanliness of the patches of the current row
  if select_motion_planning = "turndirt" [  ;; If the turndirt motion planning algorithm is selected
    set considered-dirty-patches patches with [  ;; Set the set considered-dirty-patches to be the set of patches in the current row with a difference of previous_dirt-level and current dirt-level that is greater than or equal to the set minimum dirt reduction or that have not jet been cleaned once and have a dirt-level that is greater than zero
      pycor > (((n - 1) * 5) - 1) and pycor <= (n * 5) - 1 and ((previous_dirt-level - dirt-level) >= min_dirtdecrease or timescleaned = 0) and dirt-level > 0  ;; (This is the part of the set patches-to-check with a dirt-level that is greater than 0)
    ]
    if any? considered-dirty-patches = true [  ;; If the set considered-dirty-patches is not empty
      set xcor_min min [pxcor] of considered-dirty-patches  ;; Find the minimum x-coordinate of the set considered-dirty-patches and assign it to xcor_min
      set xcor_max max [pxcor] of considered-dirty-patches  ;; Find the maximum x-coordinate of the set considered-dirty-patches and assign it to xcor_max
    ]
  ]
  set patches-to-check patches with [   ;; Select all the patches in the current row with a difference of previous_dirt-level and current dirt-level that is greater than or equal to the set minimum dirt reduction or that have not jet been cleaned once
    pycor > (((n - 1) * 5) - 1) and pycor <= (n * 5) - 1 and ((previous_dirt-level - dirt-level) >= min_dirtdecrease or timescleaned = 0)
  ]
  if all? patches-to-check [dirt-level = 0] [   ;; If all patches in the current row are considered clean, the row is clean
    set rowclean 1
    if select_motion_planning = "turndirt" [  ;; If the turndirt motion planning algorithm is selected
      set xcor_min 2  ;; Reset the values for xcor_min and xcor_max to 2 respectively ((width-window / 0.05) - 3)
      set xcor_max ((width-window / 0.05) - 3)
    ]
  ]
end

to clean  ;; Clean patches under the red cleaning section of the robot
  ask robots[
    if heading = 270 [  ;; If the robot is heading to the left
      if ((([dirt-level] of patch-at -2 0 > 0) and (((([previous_dirt-level] of patch-at -2 0) - ([dirt-level] of patch-at -2 0)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 0 = 0))) or (([dirt-level] of patch-at -2 1 > 0) and (((([previous_dirt-level] of patch-at -2 1) - ([dirt-level] of patch-at -2 1)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 1 = 0))) or (([dirt-level] of patch-at -2 2 > 0) and (((([previous_dirt-level] of patch-at -2 2) - ([dirt-level] of patch-at -2 2)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 2 = 0))) or (([dirt-level] of patch-at -2 -1 > 0) and (((([previous_dirt-level] of patch-at -2 -1) - ([dirt-level] of patch-at -2 -1)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 -1 = 0))) or (([dirt-level] of patch-at -2 -2 > 0) and (((([previous_dirt-level] of patch-at -2 -2) - ([dirt-level] of patch-at -2 -2)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 -2 = 0)))) and (select_motion_planning = "zigzag" or select_motion_planning = "turndirt")[   ;; If the robot cleans at least one patch and the zigzag or turndirt motion planning algorithm is selected
        set energy (energy + (powerconsumption_cleaning - powerconsumption) * (0.05 / v))  ;; The extra consumed energy for cleaning during the movement of one patch is added to the total consumed energy
        set water (water + waterconsumption * (0.05 / v))  ;; The extra consumed water for cleaning during the movement of one patch is added to the total water consumption
      ]
      if ([dirt-level] of patch-at -2 0 > 0) and (((([previous_dirt-level] of patch-at -2 0) - ([dirt-level] of patch-at -2 0)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 0 = 0)) [   ;; For each of the patches currently under the red cleaning section, it is checked if the dirt-level is greater than 0 and if the difference of previous_dirt-level and current dirt-level is greater than or equal to the set minimum dirt reduction or if the patch has not yet been cleaned once. If this is the case, the dirt-level is reduced with a number between mindirtred and maxdirtred
        ask patch-at -2 0 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at -2 1 > 0) and (((([previous_dirt-level] of patch-at -2 1) - ([dirt-level] of patch-at -2 1)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 1 = 0)) [
        ask patch-at -2 1 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at -2 2 > 0) and (((([previous_dirt-level] of patch-at -2 2) - ([dirt-level] of patch-at -2 2)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 2 = 0)) [
        ask patch-at -2 2 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at -2 -1 > 0) and (((([previous_dirt-level] of patch-at -2 -1) - ([dirt-level] of patch-at -2 -1)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 -1 = 0)) [
        ask patch-at -2 -1 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at -2 -2 > 0) and (((([previous_dirt-level] of patch-at -2 -2) - ([dirt-level] of patch-at -2 -2)) >= min_dirtdecrease) or ([timescleaned] of patch-at -2 -2 = 0)) [
        ask patch-at -2 -2 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
    ]
    if heading = 90 [   ;; Same as above, now for other heading direction (to the right)
      if ((([dirt-level] of patch-at 2 0 > 0) and (((([previous_dirt-level] of patch-at 2 0) - ([dirt-level] of patch-at 2 0)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 0 = 0))) or (([dirt-level] of patch-at 2 1 > 0) and (((([previous_dirt-level] of patch-at 2 1) - ([dirt-level] of patch-at 2 1)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 1 = 0))) or (([dirt-level] of patch-at 2 2 > 0) and (((([previous_dirt-level] of patch-at 2 2) - ([dirt-level] of patch-at 2 2)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 2 = 0))) or (([dirt-level] of patch-at 2 -1 > 0) and (((([previous_dirt-level] of patch-at 2 -1) - ([dirt-level] of patch-at 2 -1)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 -1 = 0))) or (([dirt-level] of patch-at 2 -2 > 0) and (((([previous_dirt-level] of patch-at 2 -2) - ([dirt-level] of patch-at 2 -2)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 -2 = 0)))) and (select_motion_planning = "zigzag" or select_motion_planning = "turndirt")[
        set energy (energy + (powerconsumption_cleaning - powerconsumption) * (0.05 / v))
        set water (water + waterconsumption * (0.05 / v))
      ]
      if ([dirt-level] of patch-at 2 0 > 0) and (((([previous_dirt-level] of patch-at 2 0) - ([dirt-level] of patch-at 2 0)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 0 = 0)) [
        ask patch-at 2 0 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at 2 1 > 0) and (((([previous_dirt-level] of patch-at 2 1) - ([dirt-level] of patch-at 2 1)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 1 = 0)) [
        ask patch-at 2 1 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at 2 2 > 0) and (((([previous_dirt-level] of patch-at 2 2) - ([dirt-level] of patch-at 2 2)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 2 = 0)) [
        ask patch-at 2 2 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at 2 -1 > 0) and (((([previous_dirt-level] of patch-at 2 -1) - ([dirt-level] of patch-at 2 -1)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 -1 = 0)) [
        ask patch-at 2 -1 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
      if ([dirt-level] of patch-at 2 -2 > 0) and (((([previous_dirt-level] of patch-at 2 -2) - ([dirt-level] of patch-at 2 -2)) >= min_dirtdecrease) or ([timescleaned] of patch-at 2 -2 = 0)) [
        ask patch-at 2 -2 [set previous_dirt-level dirt-level set dirt-level dirt-level - hard_to_clean * (mindirtred + random (maxdirtred - mindirtred)) set timescleaned timescleaned + 1]
      ]
    ]
  ]
end

to check  ;; Check patches under the grey sensor section of the robot
  ask robots[
    if heading = 270 [  ;; If the robot is heading to the left
      if ((([previous_dirt-level] of patch-at 2 0) - ([dirt-level] of patch-at 2 0)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at 2 1) - ([dirt-level] of patch-at 2 1)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at 2 2) - ([dirt-level] of patch-at 2 2)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at 2 -1) - ([dirt-level] of patch-at 2 -1)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at 2 -2) - ([dirt-level] of patch-at 2 -2)) >= min_dirtdecrease) [  ;; Checks for all the patches currently under the grey sensor section of the robot if the previous_dirt-level minus the current dirt-level is greater than or equal to the set minimum dirt reduction
        if [dirt-level] of patch-at 2 0 > 0 and xcor < ((width-window / 0.05) - 6) and justswitchedrow = 0[  ;; For each of the patches currently under the grey sensor section of the robot, it is checked if the dirt-level is greater than 0 and if the xcor is smaller than ((width-window / 0.05) - 6) (otherwise moving back 5 patches would not be possible for this heading direction) and if the robot has not just switched rows
          set xcor xcor + 5  ;; Increase the x-coordinate of the robot with 5 (for this heading direction, this amounts to moving the robot back 5 patches)
          set t (t + 5 * (0.05 / v))  ;; Add the time for moving back five patches to the total cleaning time
          set energy (energy + powerconsumption * 5 * (0.05 / v))  ;; Add the energy used for moving back five patches to the total consumed energy
          set checkzigzag 1   ;; Set the variable that checks if a zigzag motion has been initiated during the last time that the row was cleaned to 1
        ]
        if [dirt-level] of patch-at 2 1 > 0 and xcor < ((width-window / 0.05) - 6) and justswitchedrow = 0[
          set xcor xcor + 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
        if [dirt-level] of patch-at 2 2 > 0 and xcor < ((width-window / 0.05) - 6) and justswitchedrow = 0[
          set xcor xcor + 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
        if [dirt-level] of patch-at 2 -1 > 0 and xcor < ((width-window / 0.05) - 6) and justswitchedrow = 0[
          set xcor xcor + 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
        if [dirt-level] of patch-at 2 -2 > 0 and xcor < ((width-window / 0.05) - 6) and justswitchedrow = 0[
          set xcor xcor + 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
      ]
    ]
    if heading = 90 [   ;; Same as above, now for other heading direction (to the right)
      if ((([previous_dirt-level] of patch-at -2 0) - ([dirt-level] of patch-at -2 0)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at -2 1) - ([dirt-level] of patch-at -2 1)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at -2 2) - ([dirt-level] of patch-at -2 2)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at -2 -1) - ([dirt-level] of patch-at -2 -1)) >= min_dirtdecrease) and ((([previous_dirt-level] of patch-at -2 -2) - ([dirt-level] of patch-at -2 -2)) >= min_dirtdecrease) [  ;; Checks for all the patches currently under the grey sensor section of the robot if the previous_dirt-level minus the current dirt-level is greater than or equal to the set minimum dirt reduction
        if [dirt-level] of patch-at -2 0 > 0 and xcor > 5 and justswitchedrow = 0[ ;; For each of the patches currently under the grey sensor section of the robot, it is checked if the dirt-level is greater than 0 and if the xcor is bigger than 5 (otherwise moving back 5 patches would not be possible for this heading direction) and if the robot has not just switched rows
          set xcor xcor - 5  ;; Decrease the x-coordinate of the robot with 5 (for this heading direction, this amounts to moving the robot back 5 patches)
          set t (t + 5 * (0.05 / v))  ;; Add the time for moving back five patches to the total cleaning time
          set energy (energy + powerconsumption * 5 * (0.05 / v))  ;; Add the energy used for moving back five patches to the total consumed energy
          set checkzigzag 1   ;; Set the variable that checks if a zigzag motion has been initiated during the last time that the row was cleaned to 1
        ]
        if [dirt-level] of patch-at -2 1 > 0 and xcor > 5 and justswitchedrow = 0[
          set xcor xcor - 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
        if [dirt-level] of patch-at -2 2 > 0 and xcor > 5 and justswitchedrow = 0[
          set xcor xcor - 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
        if [dirt-level] of patch-at -2 -1 > 0 and xcor > 5 and justswitchedrow = 0[
          set xcor xcor - 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
        if [dirt-level] of patch-at -2 -2 > 0 and xcor > 5 and justswitchedrow = 0[
          set xcor xcor - 5
          set t (t + 5 * (0.05 / v))
          set energy (energy + powerconsumption * 5 * (0.05 / v))
          set checkzigzag 1
        ]
      ]
    ]
  ]
end

to import_world  ;; If the Import Window button is clicked, set back the last generated dirt-configuration together with other settings that have been stored in the file window.csv
  import-world "window.csv"
end
@#$#@#$#@
GRAPHICS-WINDOW
179
10
837
669
-1
-1
10.0
1
10
1
1
1
0
1
1
1
0
64
0
64
0
0
1
ticks
30.0

BUTTON
7
76
179
109
Setup Window
setup-window
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
7
10
179
43
width-window
width-window
0.75
4
3.25
0.25
1
m
HORIZONTAL

SLIDER
7
43
179
76
height-window
height-window
0.75
4
3.25
0.25
1
m
HORIZONTAL

SLIDER
7
108
179
141
dirt-intensity
dirt-intensity
0
100
100.0
1
1
%
HORIZONTAL

BUTTON
7
352
179
385
Clean Window
clean-window
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
7
207
179
240
Add Dirt
setup-dirt
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
7
240
179
274
Place Robot
setup-robot
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
7
141
179
174
nr-of-dirt-clusters
nr-of-dirt-clusters
0
10
10.0
1
1
NIL
HORIZONTAL

MONITOR
7
418
179
463
time
t
17
1
11

MONITOR
7
508
179
553
water consumption
water
17
1
11

MONITOR
7
463
179
508
energy consumption
energy
17
1
11

SWITCH
7
174
179
207
hard_to_clean_active
hard_to_clean_active
0
1
-1000

BUTTON
7
385
179
418
Import Window
import_world
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
7
307
179
352
select_motion_planning
select_motion_planning
"standard" "zigzag" "turndirt"
2

SLIDER
7
274
179
307
min_dirtdecrease
min_dirtdecrease
0.1
10
1.0
0.1
1
NIL
HORIZONTAL

@#$#@#$#@
For information, see the section in the report about the model
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

windowcleaner
true
14
Rectangle -7500403 true false 30 30 270 270
Rectangle -2674135 true false 0 0 300 60
Rectangle -1 true false 0 60 300 240
Rectangle -7500403 true false 0 240 0 240
Rectangle -7500403 true false 0 240 300 300

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
